#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
#
#    This file is part of ShoppingMortel.
#    Copyright (C) 2012-today zAmis Association
#
#    ShoppingMortel is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public License
#    as published by the Free Software Foundation, either version 3 of
#    the License, or (at your option) any later version
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details
#
#    You should have received a copy of the GNU Affero General Public
#    License along with this program.  If not, see
#    <http://www.gnu.org/licenses/>
#
# ----------------------------------------------------------------------

# Imports
# ----------------------------------------------------------------------

# Standard libraries
import argparse
import logging
import math

# Misc library
from misc.configuration										import *
from misc.general_functions									import *

# Internal libraries
import base.descriptions as descriptions
import base.shell as shell

# Functions
# ----------------------------------------------------------------------

#
# Main method
#
def ShoppingMortel_main():
    
    # Optparse & options management
    # ------------------------------------------------------------------

    # Define general options
    parser = argparse.ArgumentParser(add_help=True,
        description = "%s %s" % (program_dict['name'], program_dict['version']))
    parser.add_argument("-v", "--verbosity", type=int, dest="verbosity",
        metavar="V", choices=[0, 1, 2, 3], default=1,
        help="level of information data logged, related to logging. "\
             "0 refers to critical and error; 1 to warning; 2 to info; "\
             "3 to debug.")
    # Parse
    args = parser.parse_args()
    
    # Logging
    # ------------------------------------------------------------------
    
    # Initialize logging
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    # create console handler with a higher log level
    sh = logging.StreamHandler()
    sh.setLevel(logging.DEBUG)
    # create formatter and add it to the handlers
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    sh.setFormatter(formatter)
    # add the handlers to the logger
    logger.addHandler(sh)

    # log launch
    logger.info("%s %s launched (verbosity: %d)" % (program_dict['name'], program_dict['version'], args.verbosity))

    # Initialize world
    # ------------------------------------------------------------------

    # Create shell
    sh = shell.Shell('main shell')
    logger.info("%s created" % (sh))
    ## Create shell
    #sh = zShell('')

    ## Create zObjects
    #zObjMap_create_and_init(sh)

    ## Create zMap
    #zMap_create_and_init(sh, 'Bornsville', 'Une très chouette ville. Sincèrement.')

    ## Create zPlyr
    #zPlyr_create_and_init(sh)
    #zMap_misc_plyr_set_random_room(sh)

    ## Interfaces
    #zPlyrItf_add_shell_functions(sh)

    # Run
    # ------------------------------------------------------------------

    # Interactive mode
    #sh.zShell_exec_cmd_str('module_set zPlyr')
    #sh.zShell_exec_cmd_str('module_print')
    #sh.zShell_run()
    logger.info("Going into interactive mode")
    sh.shell_run()


#
# If launched from outside
#
if __name__ == '__main__':
    ShoppingMortel_main()
