#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
#
#    This file is part of ShoppingMortel.
#    Copyright (C) 2012-today zAmis Association
#
#    ShoppingMortel is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public License
#    as published by the Free Software Foundation, either version 3 of
#    the License, or (at your option) any later version
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details
#
#    You should have received a copy of the GNU Affero General Public
#    License along with this program.  If not, see
#    <http://www.gnu.org/licenses/>
#
# ----------------------------------------------------------------------

# Imports
# ----------------------------------------------------------------------

# Standard libraries
import argparse
import logging

# Misc library
from misc.configuration										import *
from misc.general_functions									import *

# Vars
# ----------------------------------------------------------------------

global _SHELL_NBR
_SHELL_NBR = 0

_logger = logging.getLogger(__name__)

# Class
# ----------------------------------------------------------------------

class ShellCommand():
	"""	ShellCommand class.
	"""

	def __init__(self, name, call, description, func):
		self.name = name
		self.call = call
		self.description = description
		self.func = func
	
	def __str__(self):
		return "<ShellCommand> (%s)" % (self.name)


class Shell():
	"""	Shell class.
	"""

	def __init__(self, module = ''):
		global _SHELL_NBR
		_SHELL_NBR += 1
		# ID
		self.id = _SHELL_NBR
		# status
		self.status = 'raw_input'
		# modules
		self.module = module
		self.module_stack = []
		# function pointers
		self.func_dict = {}
		# init shell cmds
		self.shell_cmd_add_default()

	def __str__(self):
		return "<Shell (id %d)>" % (self.id)

	# Modules
	# ------------------------------------------------------------------
	
	# Jump into/Come back from module
	def shell_module_stack(self, new_module):
		self.module_stack.append(self.module)
		self.module = new_module

	def shell_module_pop(self):
		self.module = self.module_stack.pop()

	# Command management
	# ------------------------------------------------------------------

	def shell_cmd_exit(self):
		print '--> exiting shell'
		self.status = 'exit'

	def _shell_cmd_print(self):
		str = ''
		str += 'exit: %s\n' % (self.func_descr['exit'])
		str += 'module\nmodule_stack: %s, module_set: %s, module_pop: %s, module_print: %s\n' % (self.func_descr['module_stack'], self.func_descr['module_set'], self.fct_descr['module_pop'], self.fct_descr['module_print'])
		str += 'commands\ncmd_print: %s' % (self.func_descr['cmd_print'])
		print str

	def _shell_cmd_module_stack(self):
		# new_module = prs_arg_lst.pop(0)
		# print '--> setting module to %s' % (new_module)
		# self.zShell_module_stack(new_module)
		pass

	def _shell_cmd_module_pop(self):
		# self.zShell_module_pop()
		# print '--> setting module to old %s' % (self.module)
		pass

	def _shell_cmd_module_print(self):
		print '--> module: %s' % (self.module)

	# Add cmd to the fct pointers dict
	def shell_cmd_add(self, func, command_name, command_descr):
		if self.func_dict.has_key(command_name):
			_logger.warning('command %s already existing' % (command_name))
		self.func_dict[command_name] = (func, command_descr)

	def shell_cmd_add_default(self):
		self.shell_cmd_add(shell_cmd_exit, 'exit', 'exit shell')
		self.shell_cmd_add(_shell_cmd_print, 'help', 'print commands')
		self.shell_cmd_add(_shell_cmd_module_stack, 'module_stack', 'module_stack')
		self.shell_cmd_add(_shell_cmd_module_pop, 'module_pop', 'module_pop')
		self.shell_cmd_add(_shell_cmd_module_print, 'module_print', 'module_print')

	# Main cmds
	# ------------------------------------------------------------------

	def shell_run(self):
		global status
		status = 'raw_input'
		while(status != 'exit'):
			# get user input
			user_input = raw_input('> ')
			# analyse input
			self.shell_exec_cmd_str(user_input)

	def shell_exec_cmd_str(self, cmd_str):
		# analyse input
		(cmd, mod_cmd, prs_arg_lst, prs_arg_dict) = self.shell_cmd_parse(cmd_str)
		# exec cmd
		self.shell_exec_cmd(cmd, mod_cmd, prs_arg_lst, prs_arg_dict)

	def shell_exec_cmd(self, cmd, mod_cmd, prs_arg_lst, prs_arg_dict):
		if self.func_dict.has_key(mod_cmd):
			print '--> exec cmd: %s' % (mod_cmd)
			self.func_dict[mod_cmd](*prs_arg_lst, **prs_arg_dict)
		else:
			print '--> cmd not found: %s or %s' % (cmd, mod_cmd)
	
	
	# Parser
	# ----------------------------------------------------------------------
	
	def shell_cmd_parse(self, user_input):
		
		# vars
		# --------------------------------------------------
		
		# global
		global status
		
		# parser vars
		prs_meta_status = 'out'
		prs_meta_arg = ''
		prs_status = 'out'
		prs_arg_dict = {}
		prs_arg_lst = []
		prs_cur_word = ''
		
		
		# sanitize cmd
		# --------------------------------------------------
		user_input = user_input.strip(' ').strip('\n')
		
		
		# parse
		# --------------------------------------------------
		for i in user_input:
			if prs_status == 'arg':
				if i == ' ':
					prs_status = 'end_arg'
				elif i == '-':
					prs_status = 'opt1'
				else:
					prs_cur_word += i
			elif prs_status == 'opt1':
				if i == '-':
					prs_status = 'opt2'
				elif i == ' ':
					prs_cur_word += '-'
					prs_status = 'end_arg'
				else:
					prs_cur_word += '-'
					prs_cur_word += i
					prs_status = 'arg'
			elif prs_status == 'opt2':
				if i == ' ':
					prs_status = 'end_opt2'
				else:
					prs_cur_word += i
			elif prs_status == 'in_guill':
				if i == '"':
					prs_status = 'end_guill'
				else:
					prs_cur_word += i
			else:
				if i not in [' ', '"', '-']:
					prs_status = 'arg'
					prs_cur_word += i
				elif i == '-':
					prs_status = 'opt1'
				elif i == '"':
					prs_status = 'in_guill'
			
			if prs_status == 'end_arg' or prs_status == 'end_guill':
				if prs_meta_status == 'in_opt':
					prs_arg_dict[prs_meta_arg] = prs_cur_word
					prs_meta_status = 'out'
				else:
					prs_arg_lst.append(prs_cur_word)
				prs_cur_word = ''
				prs_status = 'out'
			elif prs_status == 'end_opt2':
				prs_meta_arg = prs_cur_word
				prs_meta_status = 'in_opt'
				prs_cur_word = ''
				prs_status = 'out'
			
		if prs_status == 'arg':
			if prs_meta_status == 'in_opt':
				prs_arg_dict[prs_meta_arg] = prs_cur_word
				prs_meta_status = 'out'
			else:
				prs_arg_lst.append(prs_cur_word)
		
		# extract cmd
		cmd = prs_arg_lst.pop(0)
		mod_cmd = self.module + '__' + cmd
		mod_cmd = mod_cmd.lstrip('_')
		
		# add zShell
		prs_arg_dict['z_shell'] = self
		
		# return parsed data
		return (cmd, mod_cmd, prs_arg_lst, prs_arg_dict)
