#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
#
#    This file is part of ShoppingMortel.
#    Copyright (C) 2012-today zAmis Association
#
#    ShoppingMortel is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public License
#    as published by the Free Software Foundation, either version 3 of
#    the License, or (at your option) any later version
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details
#
#    You should have received a copy of the GNU Affero General Public
#    License along with this program.  If not, see
#    <http://www.gnu.org/licenses/>
#
# ----------------------------------------------------------------------

# Imports
# ----------------------------------------------------------------------


# Vars
# ----------------------------------------------------------------------

global _MAP_MAP_NBR
_MAP_MAP_NBR = 0

global _MAP_ROOM_TPL_NBR
_MAP_ROOM_TPL_NBR = 0
global _MAP_ROOM_LINK_TPL_NBR
_MAP_ROOM_LINK_TPL_NBR = 0
global _MAP_ROOM_NBR
_MAP_ROOM_NBR = 0
global _MAP_ROOM_LINK_NBR
_MAP_ROOM_LINK_NBR = 0


# Map
# ----------------------------------------------------------------------

class Map(object):
	""" Map class
    """
    
    def __init__(self, name, descr):
        """ Constructor. """
        global _MAP_MAP_NBR
        _MAP_MAP_NBR += 1
        # descr
        self.id = _MAP_ZMAP_NBR
        self.name = name
        self.descr = descr
        # buildings
        self.buildings = []

    def __str__(self):
        return "<Map (id %d)> (%s)" % (self.id, self.name)

    def map_add_building(self, building):
        """ Adds a building to the list of the map.
            
            :param building: a Building object.
        """
        self.buildings.append(building)

# Rooms
# ----------------------------------------------------------------------

class RoomTemplate(object):
    """ RoomTemplate class
        Models a template of a room. Rooms will be instanciated based
        on templates. This allows to define some typical rooms and
        (re)use them easily in the code.
    """

    def __init__(self, name, description, size, object_templates):
         """ Constructor.
            :param size: tuple(size_x, size_y, size_z)
         """
        global _MAP_ROOM_TPL_NBR
        _MAP_ROOM_TPL_NBR += 1
        # descr
        self.id = _MAP_ROOM_TPL_NBR
        self.name = name
        self.description = description
        self.size = size
        # objects templates
        self.object_templates = list(object_templates)

    def __str__(self):
        return "<RoomTemplate (id %d)> (%s)" % (self.id, self.name)


class RoomLinkTemplate(object):
	""" RoomLinkTemplate class
        Models a template for links between rooms. Like for room
        templates, room links will be instanciated based on templates.
        This allows to define some typical links between template
        rooms, and efficiently build our building.
    """
	
	def __init__(self, description, room_tpl_src, room_tpl_dst, pos_src, pos_dst, object_templates):
         """ Constructor.
            :param pos_src: tuple(x, y, z)
            :param pos_dst: tuple(x, y, z)
         """
		global _MAP_ROOM_LINK_TPL_NBR
		_MAP_ROOM_LINK_TPL_NBR += 1
		# descr
		self.id = _MAP_ROOM_LINK_TPL_NBR
		self.description = description
		self.room_tpl_src = room_tpl_src
		self.room_tpl_dst = room_tpl_dst
		# link
		self.pos_src = pos_src
		self.pos_dst = pos_dst
		# objects templates
		self.object_templates = list(object_templates)
    
	def __str__(self):
        return "<RoomLinkTemplate (id %d)> between (%s) and (%s)" % (self.id, self.z_room_tpl_src, self.z_room_tpl_dst)

class Room(object):
	""" Room class
    """
    
	def __init__(self, name, room_template, position, dust):
        """ Constructor. Instantiates a room based on a room template.

            :param pos: list[x, y, z]
            :param dust: dust state
         """
		global _MAP_ROOM_NBR
		_MAP_ROOM_NBR += 1
		# descr
		self.id = _MAP_ROOM_NBR
		self.name = name if name else room_tpl.name
		self.room_template = room_template
		self.dust = dust
		# pos
		self.position = position
		self.position_topright = Position.position_add_to_new(self.position, room_template.size)
		# room links
		self.room_src_links = []
		self.room_dst_links = []
		# objects
		self.objects = []
		#for z_obj_tpl in self.z_room_tpl.z_obj_tpl_lst:
			#self.z_obj_lst.append(zObj(z_obj_tpl))
		# building
		self.building = None
	
	def __str__(self):
		return "<Room (id %d)> (%s) @ %s" % (self.id, self.name, self.z_pos)

	def room_link_to_building(self, building):
		self.building = building
		building.building_add_room(self)
    
	@staticmethod
	def room_create(name, room_template, state, pos, building):
		room = Room(name, room_template, state, pos)
		room.room_link_to_building(building)
		for object_template in room_template.object_templates:
			#print z_obj_tpl
			pass
		return room
	
	def room_get_description(self):
		str = 'Il s\'agit de la pièce %s. %s. Cette pièce a l\'air %s.' % (self.name, self.room_template.descr, get_dust_description(self.dust))
		# links
		str += '\nIl y a %d sortie(s) dans cette pièce.' % (len(self.room_src_links) + len(self.room_dst_links))
		# objects
		for object in self.objects:
			str += '\n%s' % (object)
		return str
	
	def zRoom_pos_is_inside(self, z_pos):
		return zPos_is_contained_xy(z_pos, self.z_pos, self.z_pos_tr)
	
	def zRoom_pos_is_link(self, z_pos):
		for zr_lnk in self.z_room_lnk_src_lst:
			if zr_lnk.z_pos_src.zPos_is_equal(z_pos): return (zr_lnk, zr_lnk.z_pos_dst, zr_lnk.z_room_dst)
		for zr_lnk in self.z_room_lnk_dst_lst:
			if zr_lnk.z_pos_dst.zPos_is_equal(z_pos): return (zr_lnk, zr_lnk.z_pos_src, zr_lnk.z_room_src)
		return (False, None, None)


class RoomLink():
	
	# Constructor
	# --------------------------------------------------------------------
	
	def __init__(self, z_room_lnk_tpl, z_room_src, z_room_dst, z_pos_src, z_pos_dst):
		global _MAP_RLNK_NBR
		
		# descr
		self.id = _MAP_RLNK_NBR
		self.z_room_src = z_room_src
		self.z_room_src.z_room_lnk_src_lst.append(self)
		self.z_room_dst = z_room_dst
		self.z_room_dst.z_room_lnk_dst_lst.append(self)
		self.z_room_lnk_tpl = z_room_lnk_tpl
		# pos
		self.z_pos_src = z_pos_src
		self.z_pos_dst = z_pos_dst
		
		# misc
		_MAP_RLNK_NBR += 1
	
	def __str__(self):
		return "<zRoomLnk (id %d)>: between (%s) and (%s)" % (self.id, self.z_room_src, self.z_room_dst)
